module.exports = PanoramaModule = {
    Panorama: {
        //#region properties
        // Global variables to store configuration
        coordinates: null,
        yaw: null,
        iFrameRequestUrl: null,
        panoramaContainer: null,
        iFrameElement: null,
        panoramaClickCheckBox: null,
        closeButton: null,
        keyboardMovementEnabled: null,
        mapClickEnabled: null,
        markerDisabled: null,
        markerImage: null,
        markerCenterImage: null,
        panoramaLoaded: false,
        orientation: null,
        ttDisable: null,
        ttRight: null,
        ttBottom: null,
        iconAlignment: null,
        imageChangeEvent: null,
        loadEvent: null,
        recordingsVisible: null,
        // Maplibre
        mapLibreMap: null,
        maplibreMarker: null,
        maplibreMarkerCenter: null,
        // Openlayers
        openLayersMap: null,
        openLayersMarker: null,
        oepnLayersMarkerCenter: null,
        //#endregion
        // Get
        GetOrientation() {
            return PanoramaModule.Panorama.orientation;
        },
        GetCoordinates() {
            return PanoramaModule.Panorama.coordinates;
        },
        /**
         * @param {number} x The x coordinate, given in epsg:4326
         * @param {number} y The y coordinate, given in epsg:4326
         */
        // Opens the panorama at the location x,y
        OpenPanoramaOnLocation: function (params) {
            // If panorama is closed, open it
            if (PanoramaModule.Panorama.panoramaContainer.style.display == 'none') {
                // If imageId is given
                if (params.imageId) {
                    // Open panorama with imageId
                    PanoramaModule.Panorama.OpenPanoramaFrame(null, params.imageId, params.orientation);
                }
                else {
                    PanoramaModule.Panorama.OpenPanoramaFrame([params.x, params.y], null, params.orientation);
                }
                // If panorama is not loaded yet
                if (PanoramaModule.Panorama.panoramaLoaded === false) {
                    // Repeat this function until panorama is loaded
                    timer = setTimeout(function () {
                        PanoramaModule.Panorama.OpenPanoramaOnLocation(params);
                    }, 50);
                }
            }
            // If panorama is not loaded yet
            else if (PanoramaModule.Panorama.panoramaLoaded === false) {
                // Repeat this function until panorama is loaded
                timer = setTimeout(function () {
                    PanoramaModule.Panorama.OpenPanoramaOnLocation(params);
                }, 50);
            }
            // If panorama is loaded
            else {
                // If keyboard movement is enabled
                if (PanoramaModule.Panorama.keyboardMovementEnabled == true) {
                    // Send this message once
                    PanoramaModule.Panorama.keyboardMovementEnabled = false;
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
                }
                if (PanoramaModule.Panorama.ttDisable) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ ttDisable: true }, '*');
                }
                if (PanoramaModule.Panorama.ttRight) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ ttRight: true }, '*');
                }
                if (PanoramaModule.Panorama.ttBottom) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ ttBottom: true }, '*');
                }
                // When the frame is loaded post message to open new coordinates
                if (params.x && params.y) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ coordinates: [params.x, params.y] }, '*');
                }
                else if (params.imageId) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ imageId: params.imageId, orientation: params.orientation }, '*');
                }
            }
        },
        /**
         * @param {[number, number]} [x.y] The coordinates, given in epsg:4326 ex: [29,41]
         * @param {object} y The orientation, ex: {pitch: 25, yaw: 180, fov: 180}
         */
        // Opens the panorama iframe
        OpenPanoramaFrame: function (coordinates, imageId, orientation) {
            // If source empty fill url
            if (PanoramaModule.Panorama.iFrameElement.src == window.location.href || PanoramaModule.Panorama.iFrameElement.src == undefined) {
                PanoramaModule.Panorama.iFrameElement.src = `${PanoramaModule.Panorama.iFrameRequestUrl}`;
                if (coordinates) {
                    PanoramaModule.Panorama.iFrameElement.src += `&coordinates=${coordinates[0]},${coordinates[1]}`;
                }
                if (imageId) {
                    PanoramaModule.Panorama.iFrameElement.src += `&imageId=${imageId}`;
                }
                if (orientation) {
                    if (orientation.yaw) {
                        PanoramaModule.Panorama.iFrameElement.src += `&yaw=${orientation.yaw}`;
                    }
                    if (orientation.pitch) {
                        PanoramaModule.Panorama.iFrameElement.src += `&pitch=${orientation.pitch}`;
                    }
                    if (orientation.fov) {
                        PanoramaModule.Panorama.iFrameElement.src += `&fov=${orientation.fov}`;
                    }
                }
            }
            // If src full send message
            else {
                if (coordinates) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ coordinates: coordinates, orientation: orientation }, '*');
                }
                else if (imageId) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ imageId: imageId, orientation: orientation }, '*');
                }
            }
            // Unhide it
            PanoramaModule.Panorama.panoramaContainer.style.display = '';
            // Add event listener to close button
            PanoramaModule.Panorama.closeButton.addEventListener('click', PanoramaModule.Panorama.HidePanoramaFrame);
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            const markerCenter = document.getElementById('panorama-view-cone-center');
            // If there is a marker
            if (marker != null) {
                // Enable marker
                marker.style.display = '';
            }
            // If there is a center marker
            if (markerCenter != null) {
                // Enable marker
                markerCenter.style.display = '';
            }
        },

        // Hides the panorama frame
        HidePanoramaFrame: function (event) {
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            const markerCenter = document.getElementById('panorama-view-cone-center');
            // If there is a marker
            if (marker != null) {
                // Remove marker
                marker.style.display = 'none';
            }
            // If there is a center marker
            if (markerCenter != null) {
                // Enable marker
                markerCenter.style.display = 'none';
            }
        },

        //#region maplibre
        /**
         * @param {Object} map The map object, after initialized using: new maplibregl.Map
         * @param {Object} maplibregl The maplibregl object that is imported
         */
        // Configure MapLibre map for panorama usage, call this method in map.onload
        ConfigureMapLibreMap: function (map, maplibregl) {
            PanoramaModule.Panorama.mapLibreMap = map;
            // If keyboard movement is on
            if (PanoramaModule.Panorama.keyboardMovementEnabled == true) {
                // Disable map keyboard controls
                map.keyboard.disable();
            }
            // If map is clickable
            if (PanoramaModule.Panorama.mapClickEnabled != false) {
                // Open panorama on click location
                map.on('click', PanoramaModule.Panorama.MapLibreClickEvent);
            }
            // If marker is enabled (exists)
            if (PanoramaModule.Panorama.markerImage != null) {
                // Create new marker
                const newMarker = document.createElement('div');
                newMarker.id = 'panorama-view-cone';
                const img = new Image();
                img.addEventListener("load", function () {
                    newMarker.style.width = img.naturalWidth + 'px';
                    newMarker.style.height = img.naturalHeight + 'px';
                    newMarker.style.backgroundImage = 'url(' + img.src + ')';
                });
                img.src = PanoramaModule.Panorama.markerImage;
                PanoramaModule.Panorama.maplibreMarker = new maplibregl.Marker({ element: newMarker, pitchAlignment: PanoramaModule.Panorama.iconAlignment });
                // If there is a center
                if (PanoramaModule.Panorama.markerCenterImage) {
                    // Create new marker
                    const newMarker = document.createElement('div');
                    newMarker.id = 'panorama-view-cone-center';
                    const img = new Image();
                    img.addEventListener("load", function () {
                        newMarker.style.width = img.naturalWidth + 'px';
                        newMarker.style.height = img.naturalHeight + 'px';
                        newMarker.style.backgroundImage = 'url(' + img.src + ')';
                    });
                    img.src = PanoramaModule.Panorama.markerCenterImage;
                    PanoramaModule.Panorama.maplibreMarkerCenter = new maplibregl.Marker({ element: newMarker });
                }
            }
            // When new location or angle comes as message
            window.addEventListener('message', PanoramaModule.Panorama.MapLibreMessageEvent)
        },
        // Event for maplibre map click
        MapLibreClickEvent: function (e) {
            // If checkbox is checked
            if (PanoramaModule.Panorama.panoramaClickCheckBox.checked == true) {
                // Open panorama on location
                PanoramaModule.Panorama.OpenPanoramaOnLocation({ x: e.lngLat.lng, y: e.lngLat.lat });
            }
        },
        // Event for maplibre event-marker
        MapLibreMessageEvent: function (e) {
            if (e.data.angle != undefined && e.data.coordinates != undefined) {
                PanoramaModule.Panorama.yaw = e.data.angle;
                PanoramaModule.Panorama.coordinates = e.data.coordinates;
                // If panorama is open
                if (PanoramaModule.Panorama.panoramaContainer.style.display != 'none' && PanoramaModule.Panorama.markerDisabled !== true) {
                    // Get the marker
                    const marker = document.getElementById('panorama-view-cone');
                    // If there is a marker
                    if (marker != null) {
                        // Get its coordinates
                        const markerCoordinates = PanoramaModule.Panorama.maplibreMarker.getLngLat();
                        // If the new coordinates are different than the markers coordinates
                        if (markerCoordinates != null && markerCoordinates.lng != parseFloat(e.data.coordinates[0]) && markerCoordinates.lat != parseFloat(e.data.coordinates[1])) {
                            // Center map
                            PanoramaModule.Panorama.coordinates = e.data.coordinates;
                            PanoramaModule.Panorama.mapLibreMap.flyTo({ center: e.data.coordinates });
                        }
                    }
                    else {
                        if (PanoramaModule.Panorama.maplibreMarkerCenter) {
                            PanoramaModule.Panorama.maplibreMarkerCenter.setLngLat([29, 41]).addTo(PanoramaModule.Panorama.mapLibreMap);
                        }
                        PanoramaModule.Panorama.maplibreMarker.setLngLat([29, 41]).addTo(PanoramaModule.Panorama.mapLibreMap);
                    }
                    let bearing = PanoramaModule.Panorama.mapLibreMap.getBearing();
                    bearing = (e.data.angle - bearing + 540) % 360;
                    PanoramaModule.Panorama.maplibreMarker.setLngLat(e.data.coordinates).setRotation(bearing);
                    if (PanoramaModule.Panorama.maplibreMarkerCenter) {
                        PanoramaModule.Panorama.maplibreMarkerCenter.setLngLat(e.data.coordinates);
                    }
                }
            }
        },
        // Call this function when map is rotated
        MapLibreRotate: function () {
            // If panorama is open
            if (PanoramaModule.Panorama.panoramaContainer.style.display != 'none') {
                let bearing = PanoramaModule.Panorama.mapLibreMap.getBearing();
                bearing = (PanoramaModule.Panorama.yaw - bearing + 540) % 360;
                PanoramaModule.Panorama.maplibreMarker.setRotation(bearing);
            }
        },
        //#endregion

        //#region OpenLayers
        ConfigureOpenLayersMap: function (map, ol) {
            PanoramaModule.Panorama.openLayersMap = map;
            // If map is clickable
            if (PanoramaModule.Panorama.mapClickEnabled != false) {
                // Open panorama on click location
                map.on('singleclick', PanoramaModule.Panorama.OpenLayersClickEvent);
            }
            // If marker is enabled (exists)
            if (PanoramaModule.Panorama.markerImage != null) {
                // Add a marker with an image URL
                var marker = new ol.Overlay({
                    position: ol.proj.fromLonLat([29, 41]),
                    positioning: 'center-center',
                    element: document.createElement('div'),
                    stopEvent: false
                });
                var markerImage = new Image();
                markerImage.src = PanoramaModule.Panorama.markerImage;
                markerImage.onload = function () {
                    marker.getElement().style.backgroundImage = 'url(' + markerImage.src + ')';
                    marker.getElement().style.backgroundSize = 'contain';
                    marker.getElement().style.width = markerImage.width + 'px';
                    marker.getElement().style.height = markerImage.height + 'px';
                };
                PanoramaModule.Panorama.openLayersMarker = marker;
                map.addOverlay(marker);
                marker.getElement().setAttribute('id', 'panorama-view-cone');
                marker.getElement().style.display = 'none';
            }
            // When new location or angle comes as message
            window.addEventListener('message', PanoramaModule.Panorama.OpenLayersMessageEvent)
        },
        // Event for maplibre map click
        OpenLayersClickEvent: function (e) {
            // If checkbox is checked
            if (PanoramaModule.Panorama.panoramaClickCheckBox.checked == true) {
                // Open panorama on location
                const lonLat = ol.proj.toLonLat(e.coordinate);
                PanoramaModule.Panorama.OpenPanoramaOnLocation({ x: lonLat[0], y: lonLat[1] });
            }
        },
        OpenLayersMessageEvent: function (e) {
            if (e.data.angle != undefined && e.data.coordinates != undefined) {
                let markerCoordinates;
                // If panorama is open
                if (PanoramaModule.Panorama.panoramaContainer.style.display != 'none') {
                    // Get the marker
                    const marker = document.getElementById('panorama-view-cone');
                    // If there is a marker
                    if (marker != null) {
                        // Get its coordinates
                        markerCoordinates = PanoramaModule.Panorama.openLayersMarker.getPosition();
                        markerCoordinates = ol.proj.toLonLat(markerCoordinates);
                        // If the new coordinates are different than the markers coordinates
                        if (markerCoordinates != null && markerCoordinates[0] != parseFloat(e.data.coordinates[0]) && markerCoordinates[1] != parseFloat(e.data.coordinates[1])) {
                            // Center map
                            PanoramaModule.Panorama.coordinates = e.data.coordinates;
                            PanoramaModule.Panorama.openLayersMap.getView().setCenter(ol.proj.fromLonLat(e.data.coordinates));
                            PanoramaModule.Panorama.openLayersMarker.setPosition(ol.proj.fromLonLat(e.data.coordinates));
                        }
                    }
                    PanoramaModule.Panorama.yaw = e.data.angle;
                    PanoramaModule.Panorama.openLayersMarker.getElement().style.transform = 'rotate(' + e.data.angle + 'deg)';
                }
            }
        },

        // Toggle visibility
        ToggleRecordings: function (visibility) {
            if (PanoramaModule.Panorama.panoramaLoaded) {
                PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ recordingVisible: visibility }, '*');
            }
            else {
                setTimeout(function () {
                    PanoramaModule.Panorama.ToggleRecordings(visibility);
                }, 100);
            }
        },

        //#endregion

        /**
         * @param {String} properties.token The request token
         * @param {String} properties.environment The mode that the panorama will run on, default is deploy, test and development also available
         * @param {String} properties.panoramaType Default: 2013-2018 - The type of panorama, empty string for 2013-2018, 2018 for 2018 and 2013 for 2013
         * @param {String} properties.panoramaContainerId Default: panorama-container - The container that includes the panorama and its related controls
         * @param {String} properties.panoramaIframeId Default: panorama-iframe - Id of the iframe element
         * @param {String} properties.panoramaCloseButtonId Default: panorama-close-button - Id of the button that closes the panorama container
         * @param {Boolean} properties.mapClickEnabled Default: false - The parameter that decides whether a click on the map will open the panorama, loads default checkbox with id: "panorama-checkbox"
         * @param {String} properties.panoramaCheckBoxElementId The checkbox that enables "click map open panorama".
         * @param {String} properties.markerImage Default: https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png - The image for the view cone
         * @param {Boolean} properties.markerDisabled Default: false - If true, marker is not rendered
         * @param {Boolean} properties.markerCenterImage The image to be centered on the view cone
         * @param {Boolean} properties.keyboardControlsEnabled Default: true - Enables or disables keyboard movement inside the panorama.
         * @param {object} properties.orientation {yaw: number, pitch: number, fov: number}
         * @param {double[]} properties.coordinates {array with [x,y]}
         * @param {Boolean} properties.timeTravelRight {If time travel is to the right}
         * @param {Boolean} properties.timeTravelBottom {If time travel is to the bottom}
         * @param {Boolean} properties.timeTravelDisable {If time travel is to be disabled}
         * @param {String} properties.iconAlignment {alignment of the icon, default: auto, map or viewport}
         * @param {String} properties.imageChangeEvent {function to be triggered when image changes}
         * @param {String} properties.loadEvent {function to be triggered on first load}
         * @param {Boolean} properties.recordingsVisible {Whether the recording will be visible or not}
        */
        // Configure panorama
        ConfigurePanorama: function (properties) {
            // Set panoramaType to empty string if null
            if (properties.panoramaType === undefined) {
                properties.panoramaType = '';
            }
            if (properties.markerCenterImage) {
                PanoramaModule.Panorama.markerCenterImage = properties.markerCenterImage;
            }

            if (properties.imageChangeEvent) {
                PanoramaModule.Panorama.imageChangeEvent = properties.imageChangeEvent;
            }
            if (properties.loadEvent) {
                PanoramaModule.Panorama.loadEvent = properties.loadEvent;
            }
            if (properties.recordingsVisible !== undefined) {
                PanoramaModule.Panorama.recordingsVisible = properties.recordingsVisible;
            }

            //#region iconAlignment
            if (properties.iconAlignment) {
                PanoramaModule.Panorama.iconAlignment = properties.iconAlignment;
            }
            else {
                PanoramaModule.Panorama.iconAlignment = "auto";
            }
            //#region environment
            // Set request url according to env
            switch (properties.environment) {
                case "development":
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://localhost:44314/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
                    break;
                case "test":
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanoramatest.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
                    break;
                default:
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanorama.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
            }
            //#endregion


            //#region  panoraa container
            // Get panorama container from document
            switch (properties.panoramaContainerId) {
                case undefined:
                    PanoramaModule.Panorama.panoramaContainer = document.getElementById('panorama-container');
                    break;
                default:
                    PanoramaModule.Panorama.panoramaContainer = document.getElementById(properties.panoramaContainerId);
            }
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            //#endregion


            //#region panorama iframe
            // Get panorama iframe
            switch (properties.panoramaIframeId) {
                case undefined:
                    PanoramaModule.Panorama.iFrameElement = document.getElementById('panorama-iframe');
                    break;
                default:
                    PanoramaModule.Panorama.iFrameElement = document.getElementById(properties.panoramaIframeId);
            }
            //#endregion


            //#region close button
            // Get close button from document
            switch (properties.panoramaCloseButtonId) {
                case undefined:
                    PanoramaModule.Panorama.closeButton = document.getElementById('panorama-close-button');
                    break;
                default:
                    PanoramaModule.Panorama.closeButton = document.getElementById(properties.panoramaCloseButtonId);
            }
            //#endregion


            //#region map click 
            switch (properties.mapClickEnabled) {
                case true:
                    PanoramaModule.Panorama.mapClickEnabled = properties.mapClickEnabled;
                    switch (properties.panoramaClickCheckBox) {
                        case undefined:
                            PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById('panorama-checkbox');
                            break;
                        default:
                            PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById(properties.panoramaCheckBoxElementId);
                    }
                    switch (properties.mode) {
                        case "development":
                            PanoramaModule.Panorama.markerImage = 'https://localhost:44314/images/panoiconBlue.png';
                            break;
                        case "test":
                            PanoramaModule.Panorama.markerImage = 'https://cbspanoramatest.ibb.gov.tr/images/panoiconBlue.png';
                            break;
                        default:
                            PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                    }
                    break;
                default:
                    PanoramaModule.Panorama.mapClickEnabled = false;
            }
            //#endregion


            //#region marker image
            // If marker image is specified or the marker is not disabled
            if (properties.markerDisabled !== true) {
                switch (properties.markerImage) {
                    case undefined:
                        switch (properties.environment) {
                            case "development":
                                PanoramaModule.Panorama.markerImage = 'https://localhost:44314/images/panoiconBlue.png';
                                break;
                            case "test":
                                PanoramaModule.Panorama.markerImage = 'https://cbspanoramatest.ibb.gov.tr/images/panoiconBlue.png';
                                break;
                            default:
                                PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                        }
                        break;
                    default:
                        PanoramaModule.Panorama.markerImage = properties.markerImage;
                }
            }
            else {
                PanoramaModule.Panorama.markerDisabled = true;
            }
            switch (properties.keyboardControlsEnabled) {
                case undefined:
                    PanoramaModule.Panorama.keyboardMovementEnabled = true;
                    break;
                default:
                    PanoramaModule.Panorama.keyboardMovementEnabled = properties.keyboardControlsEnabled;
            }
            if (properties.timeTravelDisable !== undefined) {
                PanoramaModule.Panorama.ttDisable = properties.timeTravelDisable;
            }
            if (properties.timeTravelRight) {
                PanoramaModule.Panorama.ttRight = properties.timeTravelRight;
            }
            if (properties.timeTravelBottom) {
                PanoramaModule.Panorama.ttBottom = properties.timeTravelBottom;
            }
            //#endregion

            // Catch the panorama loaded message
            window.addEventListener('message', PanoramaModule.Panorama.MessageEvent);
        },
        // Event to catch load
        MessageEvent: function (e) {
            if (e.data.load === true) {
                PanoramaModule.Panorama.panoramaLoaded = true;
                if (PanoramaModule.Panorama.loadEvent) {
                    PanoramaModule.Panorama.loadEvent();
                }
                if (PanoramaModule.Panorama.recordingsVisible !== null) {
                    PanoramaModule.Panorama.ToggleRecordings(PanoramaModule.Panorama.recordingsVisible);
                }
            }
            if (e.data.angle && e.data.pitch && e.data.fov) {
                PanoramaModule.Panorama.orientation = { yaw: e.data.angle, pitch: e.data.pitch, fov: e.data.fov }
            }
            if (e.data.imageChange) {
                if (PanoramaModule.Panorama.imageChangeEvent) {
                    PanoramaModule.Panorama.imageChangeEvent();
                }
            }
        },
        // Resets this class
        Reset: function () {
            // Remove event listener
            window.removeEventListener('message', PanoramaModule.Panorama.MessageEvent);
            // If maplibre
            if (PanoramaModule.Panorama.mapLibreMap !== null) {
                window.removeEventListener('message', PanoramaModule.Panorama.MapLibreMessageEvent);
                const marker = document.getElementById('panorama-view-cone');
                if (marker) {
                    marker.parentElement.removeChild(marker);
                }
                const centerMarker = document.getElementById('panorama-view-cone-center');
                if (centerMarker) {
                    centerMarker.parentElement.removeChild(centerMarker);
                }
            }
            PanoramaModule.Panorama.iFrameElement.src = "";
            PanoramaModule.Panorama.iFrameRequestUrl = null;
            PanoramaModule.Panorama.panoramaContainer = null;
            PanoramaModule.Panorama.iFrameElement = null;
            PanoramaModule.Panorama.panoramaClickCheckBox = null;
            PanoramaModule.Panorama.closeButton = null;
            PanoramaModule.Panorama.keyboardMovementEnabled = null;
            PanoramaModule.Panorama.mapClickEnabled = null;
            PanoramaModule.Panorama.markerImage = null;
            PanoramaModule.Panorama.markerCenterImage = null;
            PanoramaModule.Panorama.mapLibreMap = null;
            PanoramaModule.Panorama.maplibreMarker = null;
            PanoramaModule.Panorama.maplibreMarkerCenter = null;
            PanoramaModule.Panorama.orientation = null;
            PanoramaModule.Panorama.yaw = null;
            PanoramaModule.Panorama.iconAlignment = null;
            PanoramaModule.Panorama.ttRight = null;
            PanoramaModule.Panorama.ttBottom = null;
            PanoramaModule.Panorama.ttDisable = null;
            PanoramaModule.Panorama.panoramaLoaded = false;
            PanoramaModule.Panorama.iconAlignment = null;
            PanoramaModule.Panorama.imageChangeEvent = null;
            PanoramaModule.Panorama.loadEvent = null;
            PanoramaModule.Panorama.recordingsVisible = null;
        }
    }
}